// tvsdkvw.cpp : implementation of the CTvsdkView class
//

#include "stdafx.h"
#include "tvsdk.h"

#include "tvsdkdoc.h"
#include "tvsdkvw.h"

#include "apturese.h"
#include "tvsettin.h"
#include "videoset.h"

#include "tvctrl32.h"

#include <vfw.h>

static HWND g_hCapture;

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTvsdkView

IMPLEMENT_DYNCREATE(CTvsdkView, CView)

BEGIN_MESSAGE_MAP(CTvsdkView, CView)
	//{{AFX_MSG_MAP(CTvsdkView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_COMMAND(IDC_SETCAPTUREFILE, OnSetcapturefile)
	ON_COMMAND(IDC_VIDEOSOURCE, OnVideosource)
	ON_COMMAND(IDC_VIDEOFORMAT, OnVideoformat)
	ON_COMMAND(IDC_COMPRESSION, OnCompression)
	ON_COMMAND(IDC_TVSETTING, OnTvsetting)
	ON_COMMAND(IDC_FULLMOTION, OnFullmotion)
	ON_COMMAND(IDC_CLIPBOARD, OnClipboard)
	ON_COMMAND(IDC_CAPTURESETTING, OnCapturesetting)
	ON_COMMAND(IDC_VIDEOSETTING, OnVideosetting)
	ON_COMMAND(IDC_SINGLEFRAME, OnSingleframe)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTvsdkView construction/destruction

CTvsdkView::CTvsdkView()
{
	m_nChannel = 4;
	m_nVolume = 15;
	m_bMute = FALSE;
	
	m_wHDelay = m_wHActive = m_wVDelay = m_wVActive = 0;
}

CTvsdkView::~CTvsdkView()
{
	TVCtrl32_Exit ();
	TVCtrl32_SetMute (TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// CTvsdkView drawing

void CTvsdkView::OnDraw(CDC* pDC)
{
	CTvsdkDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CTvsdkView diagnostics

#ifdef _DEBUG
void CTvsdkView::AssertValid() const
{
	CView::AssertValid();
}

void CTvsdkView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CTvsdkDoc* CTvsdkView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CTvsdkDoc)));
	return (CTvsdkDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CTvsdkView message handlers

int CTvsdkView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	TVCtrl32_Init ();

	m_hCapture = capCreateCaptureWindow (NULL, WS_CHILD | WS_VISIBLE, 0, 0, 320, 240, m_hWnd, 0);

	capDriverConnect (m_hCapture, 0); // Make sure you are connecting to the right driver index!

	capCaptureGetSetup (m_hCapture, &m_CaptureParms, sizeof (m_CaptureParms));
	capOverlay (m_hCapture, TRUE);   
	capPreviewScale (m_hCapture, TRUE);

	capSetCallbackOnFrame(m_hCapture, FrameCallbackProc);

	TVCtrl32_SetTVFrequency (NTSC_USA_CABLE [m_nChannel] + NTSC_USA_CABLE [0]);
	TVCtrl32_SetVolume (m_nVolume);
	TVCtrl32_SetMute (m_bMute);

	g_hCapture = m_hCapture;

	return 0;
}

void CTvsdkView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);
	::SetWindowPos (m_hCapture, NULL, 0, 0, cx, cy, SWP_NOZORDER);
}

void CTvsdkView::OnSetcapturefile()
{
    char szPath [255];

	capFileGetCaptureFile (m_hCapture, szPath, sizeof (szPath),);        

	CFileDialog dlgFile (TRUE, "AVI", szPath, OFN_HIDEREADONLY, "(VfW) *.avi |*.avi||", this);
        
	dlgFile.m_ofn.lpstrFile = (LPSTR) szPath;

	if (dlgFile.DoModal() != IDOK)
        return;
        
	capFileSetCaptureFile (m_hCapture,  (LPSTR) szPath);
}

void CTvsdkView::OnVideosource()
{
	capDlgVideoSource (m_hCapture);
}

void CTvsdkView::OnVideoformat()
{
	capDlgVideoFormat (m_hCapture);
}

void CTvsdkView::OnCompression()
{
	capDlgVideoCompression (m_hCapture);
}

void CTvsdkView::OnTvsetting()
{
	CTVSetting dlg;

	dlg.m_nChannel = m_nChannel;
	dlg.m_nVolume = m_nVolume;
	dlg.m_bMute = m_bMute;
	
	dlg.DoModal ();

	m_nChannel = dlg.m_nChannel;
	m_nVolume = dlg.m_nVolume;
	m_bMute = dlg.m_bMute;
}

void CTvsdkView::OnFullmotion()
{
	capCaptureSetSetup (m_hCapture, &m_CaptureParms, sizeof (m_CaptureParms));
	capCaptureSequence (m_hCapture);
}

void CTvsdkView::OnClipboard()
{
	capEditCopy (m_hCapture);
}

void CTvsdkView::OnCapturesetting()
{
	CaptureSetting dlg;
    
	dlg.m_dwFrameRate = (DWORD) (1E+6 / m_CaptureParms.dwRequestMicroSecPerFrame + 0.5);
	dlg.m_nTime = m_CaptureParms.wTimeLimit;
    
	if (dlg.DoModal() == IDOK)
	{
		m_CaptureParms.dwRequestMicroSecPerFrame = (DWORD) (1E+6 / dlg.m_dwFrameRate);
    	m_CaptureParms.fCaptureAudio = FALSE;
    	m_CaptureParms.fLimitEnabled = TRUE;
    	m_CaptureParms.wTimeLimit = (WORD) dlg.m_nTime;
	}
}

void CTvsdkView::OnVideosetting()
{
	CVideoSetting dlg;
	
	dlg.m_wHDelay = m_wHDelay;
	dlg.m_wHActive = m_wHActive;
	dlg.m_wVDelay = m_wVDelay;
	dlg.m_wVActive = m_wVActive;
	
	dlg.DoModal ();

	m_wHDelay = dlg.m_wHDelay;
	m_wHActive = dlg.m_wHActive;
	m_wVDelay = dlg.m_wVDelay;
	m_wVActive = dlg.m_wVActive;
}

void CTvsdkView::OnSingleframe() 
{
	capGrabFrameNoStop(m_hCapture) ;
}

LRESULT WINAPI CTvsdkView::FrameCallbackProc (HWND hWnd, LPVIDEOHDR lpVideoHdr)
{
	// The folowing codes flip the image upside down.
	CAPSTATUS CapStatus;
    capGetStatus (g_hCapture, &CapStatus, sizeof (CAPSTATUS));

	const UINT nBitPerPixel = 16; // Make sure this is the correct color depth!
	DWORD dwBytePerRow = CapStatus.uiImageWidth * nBitPerPixel / 8;
	DWORD *dwSrc, *dwDest, dwSwap;
	WORD wSize = (WORD)dwBytePerRow / 4;

	dwSrc = (DWORD*)lpVideoHdr->lpData;
	dwDest = (DWORD*)(lpVideoHdr->lpData + dwBytePerRow * (CapStatus.uiImageHeight - 1));

	for (WORD y = 0; y < CapStatus.uiImageHeight / 2; y++)
	{
		for (WORD x = 0; x < wSize; x++)
		{
			dwSwap = *dwDest;
			*dwDest++ = *dwSrc;
			*dwSrc++ = dwSwap;
		}

		dwDest -= wSize * 2;
	}
/*
	char szBuff [255];
	wsprintf (szBuff, "Frame Buffer Size = %d Bytes\nImage Width = %d, Image Height = %d",
		lpVideoHdr->dwBufferLength, CapStatus.uiImageWidth, CapStatus.uiImageHeight);
	AfxMessageBox (szBuff);
*/
	return TRUE;
}
